/* ===================================================================
   \[Xt@C
   =================================================================== */

/* ********************************************************** */
/* Include */
/* ********************************************************** */
#include "RtcSampleApl.h"


/* ********************************************************** */
/* Structure & Define */
/* ********************************************************** */
/* ^C}[ */
typedef enum {
  RTC_TIMER_STOP,
  RTC_TIMER_DOING
} RTC_TIMER_STATE;

/* RTC  */
typedef enum {
  RTC_TYPE_YEAR,
  RTC_TYPE_MONTH,
  RTC_TYPE_DAY,
  RTC_TYPE_HOUR,
  RTC_TYPE_MIN,
  RTC_TYPE_SEC
} RTC_TYPE;

/* RTC Sub Address */
#define RTC_ADR_YEAR   (0x08)
#define RTC_ADR_MONTH  (0x07)
#define RTC_ADR_DAY    (0x05)
#define RTC_ADR_HOUR   (0x04)
#define RTC_ADR_MIN    (0x03)
#define RTC_ADR_SEC    (0x02)

/* RTC Data Mask */
#define RTC_MASK_YEAR  (0xff)
#define RTC_MASK_MONTH (0x1f)
#define RTC_MASK_DAY   (0x3f)
#define RTC_MASK_HOUR  (0x3f)
#define RTC_MASK_MIN   (0x7f)
#define RTC_MASK_SEC   (0x7f)


/* bݒl */
#define RTC_TIME            (50)   /* ݒύX(50ms) */
#define RTC_INTVAL_TIME_MS  (10) 	/* Interval Timer msl */

/* ********************************************************** */
/* Local Variable */
/* ********************************************************** */
static RTC_TIMER_STATE rtc_timer_state;
static int rtc_timer_cnt;
static unsigned char now_rtc_sec;

/* ********************************************************** */
/* Local ProtoType Define */
/* ********************************************************** */
static void rtc_apl_write_rtc_info (RTC_TYPE type, RTC_APL_INFO *info);
static void rtc_apl_read_rtc_info (RTC_TYPE type, RTC_APL_INFO *info);
static void rtc_apl_set_timer (void);

/* ================================================================== */
/* ********************************************************** */
/* Src (Internal API Function) */
/* ********************************************************** */
/* ********************************************************** */
/* Tv@F@RTCAv̏                                */
/* ֐@F@RtcApl_Init                                      */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
RtcApl_Init (void)
{
  rtc_timer_state = RTC_TIMER_STOP;
  rtc_timer_cnt   = 0;

  return;
}

/* ********************************************************** */
/* Tv@F@RTCAv̎s                                  */
/* ֐@F@RtcApl_Exec                                      */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
RtcApl_Exec (void)
{
  RTC_APL_INFO info;

  if (rtc_timer_state == RTC_TIMER_STOP)
	{
	  /* ^C}[ݒ */
	  rtc_apl_set_timer ();
	  /* ^C}[쒆ݒ */
	  rtc_timer_state = RTC_TIMER_DOING;

	  /* b̒l̓ǂݍ */
	  rtc_apl_read_rtc_info (RTC_TYPE_SEC, &info);

	  /* bωĂ邩mF */
	  if (now_rtc_sec != info.sec)
		{
		  /* ̏ǂݍ */
		  rtc_apl_read_rtc_info (RTC_TYPE_YEAR, &info);
		  rtc_apl_read_rtc_info (RTC_TYPE_MONTH, &info);
		  rtc_apl_read_rtc_info (RTC_TYPE_DAY, &info);
		  rtc_apl_read_rtc_info (RTC_TYPE_HOUR, &info);
		  rtc_apl_read_rtc_info (RTC_TYPE_MIN, &info);

		  /* o */
		  RtcApl_Out_Info (&info);

		  /* ݒlݒ */
		  now_rtc_sec = info.sec;
		}
	}

  return;
}

/* ********************************************************** */
/* Tv@F@RTCAv Timer荞                          */
/* ֐@F@RtcApl_Irq_Timer                                 */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
RtcApl_Irq_Timer (void)
{
  if (rtc_timer_state == RTC_TIMER_DOING)
	{
	  if (rtc_timer_cnt <= 0)
		{
		  rtc_timer_state = RTC_TIMER_STOP;
		}
	  else
		{
		  rtc_timer_cnt--;
		}
	}

  return;
}

/* ********************************************************** */
/* Tv@F@RTCAv RTC̏                      */
/* ֐@F@RtcApl_Write_Info                                */
/* @F@RTC_APL_INFO *info : RTC̊i[             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
RtcApl_Write_Info (RTC_APL_INFO *info)
{
  rtc_apl_write_rtc_info (RTC_TYPE_YEAR, info);
  rtc_apl_write_rtc_info (RTC_TYPE_MONTH, info);
  rtc_apl_write_rtc_info (RTC_TYPE_DAY, info);
  return;
}

/* ********************************************************** */
/* Tv@F@RTCAv RTC̓ǂݍ                      */
/* ֐@F@RtcApl_Read_Info                                 */
/* @F@RTC_APL_INFO *info : RTC̊i[             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
RtcApl_Read_Info (RTC_APL_INFO *info)
{
  rtc_apl_read_rtc_info (RTC_TYPE_YEAR, info);
  rtc_apl_read_rtc_info (RTC_TYPE_MONTH, info);
  rtc_apl_read_rtc_info (RTC_TYPE_DAY, info);
  rtc_apl_read_rtc_info (RTC_TYPE_HOUR, info);
  rtc_apl_read_rtc_info (RTC_TYPE_MIN, info);
  rtc_apl_read_rtc_info (RTC_TYPE_SEC, info);

  /* ݒlݒ */
  now_rtc_sec = info->sec;
  return;
}

/* ********************************************************** */
/* Tv@F@RTCAv RTC̏o                          */
/* ֐@F@RtcApl_Out_Info                                  */
/* @F@RTC_APL_INFO *info : o͂RTC             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
RtcApl_Out_Info (RTC_APL_INFO *info)
{
#if 1
  char buf[30];

  memset (&buf[0], 0, sizeof(buf));
  sprintf (&buf[0], "%x/%x/%x %x:%x:%x\n",
		  info->year, info->month, info->day,
		   info->hour, info->min, info->sec);

  uart_xfer ("UART0", &buf[0], WR, strlen(buf));

#else
  printf ("%x/%x/%x %x:%x:%x\n",
		  info->year, info->month, info->day,
		  info->hour, info->min, info->sec);
#endif

  return;
}

/* ********************************************************** */
/* Tv@F@RTCAv Timerݒ                              */
/* ֐@F@RtcApl_Set_Timer                                 */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
void 
RtcApl_Set_Timer (void)
{
  rtc_apl_set_timer ();
  /* ^C}[쒆ݒ */
  rtc_timer_state = RTC_TIMER_DOING;

  return;
}


/* ================================================================== */
/* ********************************************************** */
/* Src (Local Function) */
/* ********************************************************** */
/* ********************************************************** */
/* Tv@F@wʂRTC̏                      */
/* ֐@F@rtc_apl_write_rtc_info                           */
/* @F@RTC_TYPE type : w                         */
/*           RTC_APL_INFO *info : i[                  */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
rtc_apl_write_rtc_info (RTC_TYPE type, RTC_APL_INFO *info)
{
  unsigned char address;
  unsigned char *data;

  switch (type)
	{
	case RTC_TYPE_YEAR:
	  address = RTC_ADR_YEAR;
	  data    = &(info->year);
	  break;
	case RTC_TYPE_MONTH:
	  address = RTC_ADR_MONTH;
	  data    = &(info->month);
	  break;
	case RTC_TYPE_DAY:
	  address = RTC_ADR_DAY;
	  data    = &(info->day);
	  break;
	case RTC_TYPE_HOUR:
	  address = RTC_ADR_HOUR;
	  data    = &(info->hour);
	  break;
	case RTC_TYPE_MIN:
	  address = RTC_ADR_MIN;
	  data    = &(info->min);
	  break;
	case RTC_TYPE_SEC:
	  address = RTC_ADR_SEC;
	  data    = &(info->sec);
	  break;
	default:
	  return;
	}

  i2c_xfer ("RTC_8564", &address, data, WR, 1);

  return;
}



/* ********************************************************** */
/* Tv@F@wʂRTC̓ǂݏo                      */
/* ֐@F@rtc_apl_read_rtc_info                            */
/* @F@RTC_TYPE type : w                         */
/*           RTC_APL_INFO *info : i[                  */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
rtc_apl_read_rtc_info (RTC_TYPE type, RTC_APL_INFO *info)
{
  unsigned char address;
  unsigned char *data;
  unsigned char mask;

  switch (type)
	{
	case RTC_TYPE_YEAR:
	  address = RTC_ADR_YEAR;
	  data    = &(info->year);
	  mask    = RTC_MASK_YEAR;
	  break;
	case RTC_TYPE_MONTH:
	  address = RTC_ADR_MONTH;
	  data    = &(info->month);
	  mask    = RTC_MASK_MONTH;
	  break;
	case RTC_TYPE_DAY:
	  address = RTC_ADR_DAY;
	  data    = &(info->day);
	  mask    = RTC_MASK_DAY;
	  break;
	case RTC_TYPE_HOUR:
	  address = RTC_ADR_HOUR;
	  data    = &(info->hour);
	  mask    = RTC_MASK_HOUR;
	  break;
	case RTC_TYPE_MIN:
	  address = RTC_ADR_MIN;
	  data    = &(info->min);
	  mask    = RTC_MASK_MIN;
	  break;
	case RTC_TYPE_SEC:
	  address = RTC_ADR_SEC;
	  data    = &(info->sec);
	  mask    = RTC_MASK_SEC;
	  break;
	default:
	  return;
	}

  i2c_xfer ("RTC_8564", &address, data, RE, 1);

  /* ǂݏoĂf[^}XNĂ */
  *data &= mask;

  return;
}

/* ********************************************************** */
/* Tv@F@RTCAṽ^C}[lݒ                        */
/* ֐@F@rtc_apl_set_timer                                */
/* @F@Ȃ                                             */
/* ߂lF@Ȃ                                             */
/* ********************************************************** */
static void 
rtc_apl_set_timer (void)
{
  rtc_timer_cnt = RTC_TIME * RTC_INTVAL_TIME_MS;  /* 50ms : l͎b */
  return;
}

